from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class ExpressoReadEmotionClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ExpressoRead",
        description="Multiclass expressive speech style classification. This is a stratfied and downsampled version of the original dataset that contains 40 hours of speech. The original dataset has two subsets - read speech and conversational speech, each having their own set of style labels. This task only includes the read speech subset.",
        reference="https://speechbot.github.io/expresso/",
        dataset={
            "path": "mteb/expresso-read-mini",
            "revision": "cf3bf160c3b18b5e0aa6607d8a724f5e397c2801",
        },
        type="AudioClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2025-01-13", "2025-01-13"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Emotion classification"],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{nguyen2023expresso,
  author = {Nguyen, Tu Anh and Hsu, Wei-Ning and d'Avirro, Antony and Shi, Bowen and Gat, Itai and Fazel-Zarani, Maryam and Remez, Tal and Copet, Jade and Synnaeve, Gabriel and Hassid, Michael and others},
  booktitle = {INTERSPEECH 2023-24th Annual Conference of the International Speech Communication Association},
  pages = {4823--4827},
  title = {Expresso: A Benchmark and Analysis of Discrete Expressive Speech Resynthesis},
  year = {2023},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "style"


class ExpressoConvEmotionClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="ExpressoConv",
        description="Multiclass expressive speech style classification. This is a stratfied and downsampled version of the original dataset that contains 40 hours of speech. The original dataset has two subsets - read speech and conversational speech, each having their own set of style labels. This task only includes the conversational speech subset.",
        reference="https://speechbot.github.io/expresso/",
        dataset={
            "path": "mteb/expresso-conv-mini",
            "revision": "fcb3c426f790493d4df74a9cf4d3109641e4de26",
        },
        type="AudioClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2025-01-13", "2025-01-13"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Emotion classification"],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@inproceedings{nguyen2023expresso,
  author = {Nguyen, Tu Anh and Hsu, Wei-Ning and d'Avirro, Antony and Shi, Bowen and Gat, Itai and Fazel-Zarani, Maryam and Remez, Tal and Copet, Jade and Synnaeve, Gabriel and Hassid, Michael and others},
  booktitle = {INTERSPEECH 2023-24th Annual Conference of the International Speech Communication Association},
  pages = {4823--4827},
  title = {Expresso: A Benchmark and Analysis of Discrete Expressive Speech Resynthesis},
  year = {2023},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "style"



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MSMARCOPL(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MSMARCO-PL",
        description="MS MARCO is a collection of datasets focused on deep learning in search",
        reference="https://microsoft.github.io/msmarco/",
        dataset={
            "path": "mteb/MSMARCO-PL",
            "revision": "a4c521ec67407f75c0df31815d2032f0e232c6a7",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2016-01-01", "2016-12-30"),  # best guess: based on publication date
        domains=["Web", "Written"],
        task_subtypes=["Question answering"],
        license="https://microsoft.github.io/msmarco/",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["MSMARCO"],
    )


class MSMARCOPLHardNegatives(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="MSMARCO-PLHardNegatives",
        description="MS MARCO is a collection of datasets focused on deep learning in search. The hard negative version has been created by pooling the 250 top documents per query from BM25, e5-multilingual-large and e5-mistral-instruct.",
        reference="https://microsoft.github.io/msmarco/",
        dataset={
            "path": "mteb/MSMARCO-PLHardNegatives",
            "revision": "a405b2712e5b06632b06636f7aeef617467d1c13",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2016-01-01", "2016-12-30"),  # best guess: based on publication date
        domains=["Web", "Written"],
        task_subtypes=["Question answering"],
        license="https://microsoft.github.io/msmarco/",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["MSMARCO"],
    )



from .sick_nl_pair_classification import SICKNLPairClassification
from .xlwic_nl_pair_classification import XLWICNLPairClassification

__all__ = [
    "SICKNLPairClassification",
    "XLWICNLPairClassification",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class MIRACLJaRetrievalLite(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MIRACLJaRetrievalLite",
        dataset={
            "path": "mteb/MIRACLJaRetrievalLite",
            "revision": "575c225da29d1f5fec01082afa56f35df0f44295",
        },
        description=(
            "MIRACL (Multilingual Information Retrieval Across a Continuum of Languages) is a multilingual "
            "retrieval dataset. This is the lightweight Japanese version with a reduced corpus (105,064 documents) "
            "constructed using hard negatives from 5 high-performance models."
        ),
        reference="https://project-miracl.github.io/",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="ndcg_at_10",
        date=("2022-06-01", "2025-01-01"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        adapted_from=["MIRACLRetrieval"],
        bibtex_citation=r"""
@article{10.1162/tacl_a_00595,
  author = {Zhang, Xinyu and Thakur, Nandan and Ogundepo, Odunayo and Kamalloo, Ehsan and Alfonso-Hermelo, David
and Li, Xiaoguang and Liu, Qun and Rezagholizadeh, Mehdi and Lin, Jimmy},
  doi = {10.1162/tacl_a_00595},
  journal = {Transactions of the Association for Computational Linguistics},
  pages = {1114-1131},
  title = {{MIRACL: A Multilingual Retrieval Dataset Covering 18 Diverse Languages}},
  volume = {11},
  year = {2023},
}

@misc{jmteb_lite,
  author = {Li, Shengzhe and Ohagi, Masaya and Ri, Ryokan and Fukuchi, Akihiko and Shibata, Tomohide
and Kawahara, Daisuke},
  howpublished = {\url{https://huggingface.co/datasets/sbintuitions/JMTEB-lite}},
  title = {{J}{M}{T}{E}{B}-lite: {T}he {L}ightweight {V}ersion of {JMTEB}},
  year = {2025},
}
""",
    )



from mteb.models import ModelMeta, SentenceTransformerEncoderWrapper

bica_base = ModelMeta(
    name="bisectgroup/BiCA-base",
    model_type=["dense"],
    loader=SentenceTransformerEncoderWrapper,
    languages=["eng-Latn"],
    open_weights=True,
    revision="31237a836e5ae908c308a256573e5f0986498574",
    release_date="2025-11-14",
    n_parameters=109482240,
    n_embedding_parameters=23_440_896,
    memory_usage_mb=418,
    embed_dim=768,
    license="mit",
    max_tokens=512,
    reference="https://huggingface.co/bisectgroup/BiCA-base",
    similarity_fn_name="cosine",
    framework=["Sentence Transformers", "PyTorch", "safetensors"],
    use_instructions=False,
    public_training_code="https://github.com/NiravBhattLab/BiCA",
    public_training_data="https://huggingface.co/datasets/bisectgroup/hard-negatives-traversal",
    adapted_from="thenlper/gte-base",
    citation="""
@misc{sinha2025bicaeffectivebiomedicaldense,
      title={BiCA: Effective Biomedical Dense Retrieval with Citation-Aware Hard Negatives},
      author={Aarush Sinha and Pavan Kumar S and Roshan Balaji and Nirav Pravinbhai Bhatt},
      year={2025},
      eprint={2511.08029},
      archivePrefix={arXiv},
      primaryClass={cs.IR},
      url={https://arxiv.org/abs/2511.08029},
}
""",
    training_datasets=set(),
)



from __future__ import annotations

from typing import TYPE_CHECKING

from mteb.models.instruct_wrapper import InstructSentenceTransformerModel
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from mteb.types import PromptType


def instruction_template(
    instruction: str, prompt_type: PromptType | None = None
) -> str:
    return "{instruction}"


vdr_languages = [
    "eng-Latn",
    "ita-Latn",
    "fra-Latn",
    "deu-Latn",
    "spa-Latn",
]

vdr_2b_multi_v1 = ModelMeta(
    loader=InstructSentenceTransformerModel,
    loader_kwargs=dict(
        instruction_template=instruction_template,
        max_seq_length=32768,
        apply_instruction_to_passages=True,
    ),
    name="llamaindex/vdr-2b-multi-v1",
    model_type=["dense"],
    languages=vdr_languages,
    open_weights=True,
    revision="2c4e54c8db4071cc61fc3c62f4490124e40c37db",
    release_date="2024-01-08",
    modalities=["text"],  # TODO: integrate with image
    n_parameters=2208985600,
    n_embedding_parameters=233_373_696,
    memory_usage_mb=4213,
    max_tokens=32768,
    embed_dim=1536,
    license="apache-2.0",
    reference="https://huggingface.co/llamaindex/vdr-2b-multi-v1",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/llamaindex/vdr-multilingual-train",
    training_datasets=set(
        # llamaindex/vdr-multilingual-train
        "VDRMultilingualRetrieval",
    ),
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

import torch
from tqdm.auto import tqdm

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType


class DINOModel(AbsEncoder):
    """A wrapper class for DINO models that supports image encoding.
    Text encoding and text-image fusion are not supported.
    """

    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        **kwargs: Any,
    ):
        from transformers import AutoImageProcessor, AutoModel

        self.model_name = model_name
        self.device = device
        self.model = AutoModel.from_pretrained(model_name, revision=revision).to(
            self.device
        )
        self.processor = AutoImageProcessor.from_pretrained(
            model_name, revision=revision
        )

    @staticmethod
    def get_text_embeddings(
        texts: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ):
        raise ValueError("DINO models only support image encoding.")

    def get_image_embeddings(
        self,
        images: DataLoader[BatchedInput],
        show_progress_bar: bool = True,
        pooling: Literal["cls", "mean"] = "cls",
        **kwargs: Any,
    ):
        all_image_embeddings = []

        with torch.no_grad():
            for batch in tqdm(
                images, disable=not show_progress_bar, desc="Image Encoding"
            ):
                inputs = self.processor(images=batch["image"], return_tensors="pt")
                inputs = {k: v.to(self.device) for k, v in inputs.items()}
                image_outputs = self.model(**inputs)
                features = image_outputs.last_hidden_state
                if pooling == "cls":
                    features = features[:, 0, :]  # TODO: confirm best practice
                elif pooling == "mean":
                    features = features.mean(dim=1)
                else:
                    raise ValueError(
                        "Pooling methods not implemented. Use cls or mean."
                    )
                all_image_embeddings.append(features.cpu())

        all_image_embeddings = torch.cat(all_image_embeddings, dim=0)
        return all_image_embeddings

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        image_embeddings = None
        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "image" in inputs.dataset.features:
            image_embeddings = self.get_image_embeddings(inputs, **kwargs)

        if text_embeddings is not None and image_embeddings is not None:
            raise ValueError("DINO models only support image encoding.")
        elif text_embeddings is not None:
            return text_embeddings
        elif image_embeddings is not None:
            return image_embeddings
        raise ValueError("No text or image data found.")


dinov2_training_datasets = set(
    # LVD-142M
    #  ImageNet-22k
)


dinov2_small = ModelMeta(
    loader=DINOModel,
    name="facebook/dinov2-small",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="ed25f3a31f01632728cabb09d1542f84ab7b0056",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=22056576,
    n_embedding_parameters=0,
    memory_usage_mb=84,
    max_tokens=None,
    embed_dim=384,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/dinov2-small",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
    citation="""@misc{oquab2023dinov2,
      title={DINOv2: Learning Robust Visual Features without Supervision},
      author={Maxime Oquab and Timothée Darcet and Théo Moutakanni and Huy Vo and Marc Szafraniec and Vasil Khalidov and Pierre Fernandez and Daniel Haziza and Francisco Massa and Alaaeldin El-Nouby and Mahmoud Assran and Nicolas Ballas and Wojciech Galuba and Russell Howes and Po-Yao Huang and Shang-Wen Li and Ishan Misra and Michael Rabbat and Vasu Sharma and Gabriel Synnaeve and Hu Xu and Hervé Jegou and Julien Mairal and Patrick Labatut and Armand Joulin and Piotr Bojanowski},
      year={2023},
      eprint={2304.07193},
      archivePrefix={arXiv},
      primaryClass={cs.CV}
}""",
)

dinov2_base = ModelMeta(
    loader=DINOModel,
    name="facebook/dinov2-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="f9e44c814b77203eaa57a6bdbbd535f21ede1415",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=86580480,
    n_embedding_parameters=0,
    memory_usage_mb=330,
    max_tokens=None,
    embed_dim=768,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/dinov2-base",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
    citation="""@misc{oquab2023dinov2,
      title={DINOv2: Learning Robust Visual Features without Supervision},
      author={Maxime Oquab and Timothée Darcet and Théo Moutakanni and Huy Vo and Marc Szafraniec and Vasil Khalidov and Pierre Fernandez and Daniel Haziza and Francisco Massa and Alaaeldin El-Nouby and Mahmoud Assran and Nicolas Ballas and Wojciech Galuba and Russell Howes and Po-Yao Huang and Shang-Wen Li and Ishan Misra and Michael Rabbat and Vasu Sharma and Gabriel Synnaeve and Hu Xu and Hervé Jegou and Julien Mairal and Patrick Labatut and Armand Joulin and Piotr Bojanowski},
      year={2023},
      eprint={2304.07193},
      archivePrefix={arXiv},
      primaryClass={cs.CV}
}""",
)

dinov2_large = ModelMeta(
    loader=DINOModel,
    name="facebook/dinov2-large",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="47b73eefe95e8d44ec3623f8890bd894b6ea2d6c",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=304368640,
    n_embedding_parameters=0,
    memory_usage_mb=1161,
    max_tokens=None,
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/dinov2-large",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
    citation="""@misc{oquab2023dinov2,
      title={DINOv2: Learning Robust Visual Features without Supervision},
      author={Maxime Oquab and Timothée Darcet and Théo Moutakanni and Huy Vo and Marc Szafraniec and Vasil Khalidov and Pierre Fernandez and Daniel Haziza and Francisco Massa and Alaaeldin El-Nouby and Mahmoud Assran and Nicolas Ballas and Wojciech Galuba and Russell Howes and Po-Yao Huang and Shang-Wen Li and Ishan Misra and Michael Rabbat and Vasu Sharma and Gabriel Synnaeve and Hu Xu and Hervé Jegou and Julien Mairal and Patrick Labatut and Armand Joulin and Piotr Bojanowski},
      year={2023},
      eprint={2304.07193},
      archivePrefix={arXiv},
      primaryClass={cs.CV}
}""",
)

dinov2_giant = ModelMeta(
    loader=DINOModel,
    name="facebook/dinov2-giant",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="611a9d42f2335e0f921f1e313ad3c1b7178d206d",
    release_date="2023-07-18",
    modalities=["image"],
    n_parameters=1136480768,
    n_embedding_parameters=0,
    memory_usage_mb=4335,
    max_tokens=None,
    embed_dim=1536,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/facebookresearch/dinov2",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/dinov2-giant",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=dinov2_training_datasets,
    citation="""@misc{oquab2023dinov2,
      title={DINOv2: Learning Robust Visual Features without Supervision},
      author={Maxime Oquab and Timothée Darcet and Théo Moutakanni and Huy Vo and Marc Szafraniec and Vasil Khalidov and Pierre Fernandez and Daniel Haziza and Francisco Massa and Alaaeldin El-Nouby and Mahmoud Assran and Nicolas Ballas and Wojciech Galuba and Russell Howes and Po-Yao Huang and Shang-Wen Li and Ishan Misra and Michael Rabbat and Vasu Sharma and Gabriel Synnaeve and Hu Xu and Hervé Jegou and Julien Mairal and Patrick Labatut and Armand Joulin and Piotr Bojanowski},
      year={2023},
      eprint={2304.07193},
      archivePrefix={arXiv},
      primaryClass={cs.CV}
}""",
)

webssl_dino_training_datasets = set(
    # MetaCLIP 2B samples
)

webssl_dino300m_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino300m-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="8529cdb3fb75014932af3b896455fc21c386168e",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=303655168,
    n_embedding_parameters=0,
    memory_usage_mb=1158,
    max_tokens=None,
    embed_dim=1024,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino300m-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino1b_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino1b-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="d3bf033d9c8cc62ea9e73c40956642cad2ec568a",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=1134771200,
    n_embedding_parameters=0,
    memory_usage_mb=4329,
    max_tokens=None,
    embed_dim=1536,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino1b-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino2b_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino2b-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="cd5893e3fd2e988eb716792049b3dd53b3f1b68b",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2084237568,
    n_embedding_parameters=0,
    memory_usage_mb=7951,
    max_tokens=None,
    embed_dim=2688,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino2b-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino3b_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino3b-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="2d015c340b16bc47bc6557fcb4e6c83a9d4aa1d3",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2948317184,
    n_embedding_parameters=0,
    memory_usage_mb=11247,
    max_tokens=None,
    embed_dim=3072,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino3b-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino5b_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino5b-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="88006b18b9af369f6c611db7a64d908bde3714e0",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=4938338816,
    n_embedding_parameters=0,
    memory_usage_mb=18838,
    max_tokens=None,
    embed_dim=3584,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino5b-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino7b_full8b_224 = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino7b-full8b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="c6085463ea680043042a80c6d41db2c65e85f466",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=6450170880,
    n_embedding_parameters=0,
    memory_usage_mb=24605,
    max_tokens=None,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino7b-full8b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino7b_full8b_378 = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino7b-full8b-378",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="53c8c5b43070bd2ddb3f66161140408ce832301f",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=6452108288,
    n_embedding_parameters=0,
    memory_usage_mb=24613,
    max_tokens=None,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino7b-full8b-378",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino7b_full8b_518 = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino7b-full8b-518",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="aee350d2c5e3e5fdb7ee6985291d808ea5eef431",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=6454729728,
    n_embedding_parameters=0,
    memory_usage_mb=24623,
    max_tokens=None,
    embed_dim=4096,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino7b-full8b-518",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)


webssl_dino2b_light2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino2b-light2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="633a663f304e63cc3cbec3f7f9ca2fbc94736128",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2084237568,
    n_embedding_parameters=0,
    memory_usage_mb=7951,
    max_tokens=None,
    embed_dim=2688,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino2b-light2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino2b_heavy2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino2b-heavy2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="9f46eb0c0129656a1ef195fde072e3765abdb7c6",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2084237568,
    n_embedding_parameters=0,
    memory_usage_mb=7951,
    max_tokens=None,
    embed_dim=2688,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino2b-heavy2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino3b_light2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino3b-light2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="4d0160f60673805431f4ad14983e712ed88be5b8",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2948317184,
    n_embedding_parameters=0,
    memory_usage_mb=11247,
    max_tokens=None,
    embed_dim=3072,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino3b-light2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_dino3b_heavy2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-dino3b-heavy2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="dd39c2910747561b332285d96c4dce0bdb240775",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=2948317184,
    n_embedding_parameters=0,
    memory_usage_mb=11247,
    max_tokens=None,
    embed_dim=3072,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-dino3b-heavy2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_mae300m_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-mae300m-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="4655a0ac1726c206ba14d5ccb26758c62a4d03b0",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=304351232,
    n_embedding_parameters=0,
    memory_usage_mb=1161,
    max_tokens=None,
    embed_dim=1024,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-mae300m-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_mae700m_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-mae700m-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="c32be382e757d73a178de1ead62c27391d4b4280",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=632404480,
    n_embedding_parameters=0,
    memory_usage_mb=2412,
    max_tokens=None,
    embed_dim=1280,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-mae700m-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)

webssl_mae1b_full2b = ModelMeta(
    loader=DINOModel,
    name="facebook/webssl-mae1b-full2b-224",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="5880aefedbad8db0f44d27358f6f08e8576f70fc",
    release_date="2025-04-24",
    modalities=["image"],
    n_parameters=1136925696,
    n_embedding_parameters=0,
    memory_usage_mb=4337,
    max_tokens=None,
    embed_dim=1536,
    license="cc-by-nc-4.0",
    open_weights=True,
    public_training_code="",
    public_training_data=None,
    framework=["PyTorch", "Transformers", "safetensors"],
    reference="https://huggingface.co/facebook/webssl-mae1b-full2b-224",
    similarity_fn_name=None,
    use_instructions=False,
    training_datasets=webssl_dino_training_datasets,
    citation="""@article{fan2025scaling,
  title={Scaling Language-Free Visual Representation Learning},
  author={David Fan and Shengbang Tong and Jiachen Zhu and Koustuv Sinha and Zhuang Liu and Xinlei Chen and Michael Rabbat and Nicolas Ballas and Yann LeCun and Amir Bar and Saining Xie},
  year={2025},
  eprint={2504.01017},
  archivePrefix={arXiv},
  primaryClass={cs.CV}
}""",
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, Literal

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_image_dependencies, requires_package
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.types import PromptType

if TYPE_CHECKING:
    from PIL import Image
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput

logger = logging.getLogger(__name__)


def _downsample_image(
    image: Image.Image, max_pixels: int = 16000000, target_longest_side: int = 4000
) -> Image.Image:
    """If image pixel > max_pixels, downsample it to target_longest_side while keeping the width height ratio.

    Returns:
        The downsampled image.
    """
    from PIL.Image import Resampling

    width, height = image.size
    pixels = width * height

    if pixels > max_pixels:
        if width > height:
            new_width = target_longest_side
            new_height = int(height * (target_longest_side / width))
        else:
            new_height = target_longest_side
            new_width = int(width * (target_longest_side / height))

        new_size = (new_width, new_height)
        logger.info(
            f"Downsampling image from {width}x{height} to {new_width}x{new_height}"
        )
        return image.resize(new_size, Resampling.LANCZOS)
    if width > height:
        if width > 10000:
            logger.error("Processing extremely wide images.")
            return image.resize((10000, height), Resampling.LANCZOS)
    else:
        if height > 10000:
            logger.error("Processing extremely high images.")
            return image.resize((width, 10000), Resampling.LANCZOS)
    return image


def voyage_v_loader(model_name, **kwargs):
    requires_package(
        voyage_v_loader,
        "voyageai",
        model_name,
        "pip install 'mteb[voyage_v]'",
    )
    requires_package(
        voyage_v_loader,
        "tenacity",
        model_name,
        "pip install 'mteb[voyage_v]'",
    )
    import voyageai
    from tenacity import retry, stop_after_attempt, wait_exponential

    class VoyageMultiModalModelWrapper(AbsEncoder):
        def __init__(
            self,
            model_name: str,
            **kwargs: Any,
        ):
            requires_image_dependencies()

            self.model_name = model_name.split("/")[-1]
            self.vo = voyageai.Client()

        @retry(
            stop=stop_after_attempt(6),  # Stop after 6 attempts
            wait=wait_exponential(multiplier=1, max=60),  # Exponential backoff
        )
        def _multimodal_embed(self, inputs, model, input_type):
            return self.vo.multimodal_embed(inputs, model=model, input_type=input_type)

        def get_text_embeddings(
            self,
            texts: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            prompt_type: PromptType | None = None,
            input_type: Literal["document", "query"] | None = None,
            **kwargs: Any,
        ):
            if input_type is None and prompt_type is not None:
                if prompt_type == PromptType.document:
                    input_type = "document"
                elif prompt_type == PromptType.query:
                    input_type = "query"

            all_text_embeddings = []

            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                batch_texts = [[text] for text in batch["text"]]

                # with retry mechanism
                embeddings = self._multimodal_embed(
                    batch_texts, model=self.model_name, input_type=input_type
                ).embeddings
                all_text_embeddings.append(torch.tensor(embeddings))
            all_text_embeddings = torch.vstack(all_text_embeddings)
            return all_text_embeddings

        def get_image_embeddings(
            self,
            images: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            prompt_type: PromptType | None = None,
            input_type: Literal["document", "query"] | None = None,
            **kwargs: Any,
        ):
            if input_type is None and prompt_type is not None:
                if prompt_type == PromptType.document:
                    input_type = "document"
                elif prompt_type == PromptType.query:
                    input_type = "query"

            all_image_embeddings = []

            for batch in tqdm(
                images, disable=not show_progress_bar, desc="Image Encoding"
            ):
                batch_images = [[_downsample_image(image)] for image in batch["image"]]
                embeddings = self._multimodal_embed(
                    batch_images, model=self.model_name, input_type=input_type
                ).embeddings
                all_image_embeddings.append(torch.tensor(embeddings))
            all_image_embeddings = torch.vstack(all_image_embeddings)
            return all_image_embeddings

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            show_progress_bar: bool = True,
            **kwargs: Any,
        ) -> Array:
            input_type = "document"  # default
            if prompt_type is not None:
                if prompt_type == PromptType.document:
                    input_type = "document"
                elif prompt_type == PromptType.query:
                    input_type = "query"

            text_embeddings = None
            image_embeddings = None

            interleaved_embeddings = []
            if "text" in inputs.dataset.features and "image" in inputs.dataset.features:
                for batch in tqdm(
                    inputs, disable=not show_progress_bar, desc="Interleaved Encoding"
                ):
                    batch_images = [
                        _downsample_image(image) for image in batch["image"]
                    ]
                    batch_texts = batch["text"]
                    interleaved_inputs = [
                        [text, image] for image, text in zip(batch_images, batch_texts)
                    ]
                    embeddings = self._multimodal_embed(
                        interleaved_inputs,
                        model=self.model_name,
                        input_type=input_type,
                    ).embeddings
                    interleaved_embeddings.append(torch.tensor(embeddings))
                interleaved_embeddings = torch.vstack(interleaved_embeddings)
                return interleaved_embeddings
            elif "text" in inputs.dataset.features:
                text_embeddings = self.get_text_embeddings(
                    inputs, input_type=input_type
                )
            elif "image" in inputs.dataset.features:
                image_embeddings = self.get_image_embeddings(
                    inputs, input_type=input_type
                )

            if text_embeddings is not None:
                return text_embeddings
            elif image_embeddings is not None:
                return image_embeddings
            raise ValueError

    return VoyageMultiModalModelWrapper(model_name, **kwargs)


voyage_v = ModelMeta(
    loader=voyage_v_loader,
    name="voyageai/voyage-multimodal-3",
    model_type=["dense"],
    languages=[],  # Unknown
    revision="1",
    release_date="2024-11-10",
    n_parameters=None,
    n_embedding_parameters=None,
    memory_usage_mb=None,
    max_tokens=32768,
    embed_dim=1024,
    license="mit",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["API"],
    modalities=["image", "text"],
    open_weights=False,
    public_training_code=None,
    public_training_data=None,
    reference="https://huggingface.co/voyageai/voyage-multimodal-3",
    use_instructions=None,
    training_datasets=set(),  # No overlap with MTEB according to Voyage, could overlap with MIEB, didn't ask
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, TypedDict

from sklearn.metrics.pairwise import (
    paired_cosine_distances,
    paired_euclidean_distances,
    paired_manhattan_distances,
)

from mteb._create_dataloaders import create_dataloader
from mteb.similarity_functions import compute_pairwise_similarity

from .evaluator import Evaluator

if TYPE_CHECKING:
    from datasets import Dataset

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.models import EncoderProtocol
    from mteb.types import EncodeKwargs, PromptType

logger = logging.getLogger(__name__)


class STSEvaluatorScores(TypedDict):
    """Scores for STS evaluation

    Attributes:
        cosine_scores: Cosine similarity scores between pairs of sentences.
        manhattan_distances: Negative Manhattan distances between pairs of sentences.
        euclidean_distances: Negative Euclidean distances between pairs of sentences.
        similarity_scores: Similarity scores computed using the model's similarity function, if available.
    """

    cosine_scores: list[float]
    manhattan_distances: list[float]
    euclidean_distances: list[float]
    similarity_scores: list[float] | None


class AnySTSEvaluator(Evaluator):
    def __init__(
        self,
        dataset: Dataset,
        sentences_column_names: tuple[str, str],
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        input1_prompt_type: PromptType | None,
        input2_prompt_type: PromptType | None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.dataset = dataset
        self.input_columns = sentences_column_names
        self.task_metadata = task_metadata
        self.hf_split = hf_split
        self.hf_subset = hf_subset
        self.input1_prompt_type = input1_prompt_type
        self.input2_prompt_type = input2_prompt_type

    def __call__(
        self,
        model: EncoderProtocol,
        *,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None = None,
    ) -> STSEvaluatorScores:
        logger.info("Running semantic similarity - Encoding samples (1/2)")
        embeddings1 = model.encode(
            create_dataloader(
                self.dataset.select_columns([self.input_columns[0]]),
                self.task_metadata,
                input_column=self.input_columns[0],
                num_proc=num_proc,
                **encode_kwargs,
            ),
            task_metadata=self.task_metadata,
            hf_split=self.hf_split,
            hf_subset=self.hf_subset,
            prompt_type=self.input1_prompt_type,
            **encode_kwargs,
        )

        logger.info("Running semantic similarity - Encoding samples (2/2)...")
        embeddings2 = model.encode(
            create_dataloader(
                self.dataset.select_columns([self.input_columns[1]]),
                self.task_metadata,
                input_column=self.input_columns[1],
                **encode_kwargs,
            ),
            task_metadata=self.task_metadata,
            hf_split=self.hf_split,
            hf_subset=self.hf_subset,
            prompt_type=self.input2_prompt_type,
            **encode_kwargs,
        )

        logger.info("Running semantic similarity - Evaluating similarity...")
        cosine_scores = 1 - (paired_cosine_distances(embeddings1, embeddings2))
        manhattan_distances = -paired_manhattan_distances(embeddings1, embeddings2)
        euclidean_distances = -paired_euclidean_distances(embeddings1, embeddings2)
        similarity_scores = compute_pairwise_similarity(model, embeddings1, embeddings2)

        logger.info("Running semantic similarity - Finished.")
        return STSEvaluatorScores(
            cosine_scores=cosine_scores.tolist(),
            manhattan_distances=manhattan_distances.tolist(),
            euclidean_distances=euclidean_distances.tolist(),
            similarity_scores=similarity_scores.tolist(),
        )



from .ambient_acoustic_context_clustering import AmbientAcousticContextClustering
from .arxiv_clustering_p2p import ArxivClusteringP2P, ArxivClusteringP2PFast
from .arxiv_clustering_s2s import ArxivClusteringS2S
from .arxiv_hierarchical_clustering import (
    ArXivHierarchicalClusteringP2P,
    ArXivHierarchicalClusteringS2S,
)
from .big_patent_clustering import BigPatentClustering, BigPatentClusteringFast
from .biorxiv_clustering_p2p import BiorxivClusteringP2P, BiorxivClusteringP2PFast
from .biorxiv_clustering_s2s import BiorxivClusteringS2S, BiorxivClusteringS2SFast
from .built_bench_clustering_p2p import BuiltBenchClusteringP2P
from .built_bench_clustering_s2s import BuiltBenchClusteringS2S
from .cifar import CIFAR10Clustering, CIFAR100Clustering
from .clus_trec_covid import ClusTrecCovid
from .crema_d_clustering import CREMADClustering
from .hume_arxiv_clustering_p2p import HUMEArxivClusteringP2P
from .hume_reddit_clustering_p2p import HUMERedditClusteringP2P
from .hume_wiki_cities_clustering import HUMEWikiCitiesClustering
from .image_net import ImageNet10Clustering, ImageNetDog15Clustering
from .medrxiv_clustering_p2p import MedrxivClusteringP2P, MedrxivClusteringP2PFast
from .medrxiv_clustering_s2s import MedrxivClusteringS2S, MedrxivClusteringS2SFast
from .reddit_clustering import RedditClustering, RedditFastClusteringS2S
from .reddit_clustering_p2p import RedditClusteringP2P, RedditFastClusteringP2P
from .stack_exchange_clustering import (
    StackExchangeClustering,
    StackExchangeClusteringFast,
)
from .stack_exchange_clustering_p2p import (
    StackExchangeClusteringP2P,
    StackExchangeClusteringP2PFast,
)
from .tiny_image_net import TinyImageNet
from .twenty_newsgroups_clustering import (
    TwentyNewsgroupsClustering,
    TwentyNewsgroupsClusteringFast,
)
from .voice_gender import VoiceGenderClustering
from .vox_celeb_clustering import VoxCelebClustering
from .vox_populi_accent_clustering import VoxPopuliAccentClustering
from .wiki_cities_clustering import WikiCitiesClustering
from .wikipedia_chemistry_specialties_clustering import (
    WikipediaChemistrySpecialtiesClustering,
)
from .wikipedia_chemistry_topics_clustering import WikipediaChemistryTopicsClustering

__all__ = [
    "AmbientAcousticContextClustering",
    "ArXivHierarchicalClusteringP2P",
    "ArXivHierarchicalClusteringS2S",
    "ArxivClusteringP2P",
    "ArxivClusteringP2PFast",
    "ArxivClusteringS2S",
    "BigPatentClustering",
    "BigPatentClusteringFast",
    "BiorxivClusteringP2P",
    "BiorxivClusteringP2PFast",
    "BiorxivClusteringS2S",
    "BiorxivClusteringS2SFast",
    "BuiltBenchClusteringP2P",
    "BuiltBenchClusteringS2S",
    "CIFAR10Clustering",
    "CIFAR100Clustering",
    "CREMADClustering",
    "ClusTrecCovid",
    "HUMEArxivClusteringP2P",
    "HUMERedditClusteringP2P",
    "HUMEWikiCitiesClustering",
    "ImageNet10Clustering",
    "ImageNetDog15Clustering",
    "MedrxivClusteringP2P",
    "MedrxivClusteringP2PFast",
    "MedrxivClusteringS2S",
    "MedrxivClusteringS2SFast",
    "RedditClustering",
    "RedditClusteringP2P",
    "RedditFastClusteringP2P",
    "RedditFastClusteringS2S",
    "StackExchangeClustering",
    "StackExchangeClusteringFast",
    "StackExchangeClusteringP2P",
    "StackExchangeClusteringP2PFast",
    "TinyImageNet",
    "TwentyNewsgroupsClustering",
    "TwentyNewsgroupsClusteringFast",
    "VoiceGenderClustering",
    "VoxCelebClustering",
    "VoxPopuliAccentClustering",
    "WikiCitiesClustering",
    "WikipediaChemistrySpecialtiesClustering",
    "WikipediaChemistryTopicsClustering",
]



from .klue_tc import KlueTC, KlueTCV2
from .kor_fin import KorFin
from .kor_hate_classification import KorHateClassification, KorHateClassificationV2
from .kor_sarcasm_classification import (
    KorSarcasmClassification,
    KorSarcasmClassificationV2,
)

__all__ = [
    "KlueTC",
    "KlueTCV2",
    "KorFin",
    "KorHateClassification",
    "KorHateClassificationV2",
    "KorSarcasmClassification",
    "KorSarcasmClassificationV2",
]



from mteb.abstasks import AbsTaskZeroShotClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SpeechCommandsZeroshotClassificationV01(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="SpeechCommandsZeroshotv0.01",
        description="Sound Classification/Keyword Spotting Dataset. This is a set of one-second audio clips containing a single spoken English word or background noise. These words are from a small set of commands such as 'yes', 'no', and 'stop' spoken by various speakers. With a total of 10 labels/commands for keyword spotting and a total of 30 labels for other auxiliary tasks",
        reference="https://huggingface.co/datasets/google/speech_commands",
        dataset={
            "path": "mteb/SpeechCommandsZeroshotv0.01",
            "revision": "200a38f4d145460758244d6613cb08dffe70fa8a",
        },
        type="AudioZeroshotClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2018-07-07", "2018-07-13"),
        domains=["Spoken"],
        task_subtypes=["Keyword Spotting"],
        license="cc-by-4.0",  # Replace with appropriate license from allowed list
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio", "text"],
        sample_creation="found",
        bibtex_citation=r"""
@article{DBLP:journals/corr/abs-1804-03209,
  author = {Pete Warden},
  bibsource = {dblp computer science bibliography, https://dblp.org},
  biburl = {https://dblp.org/rec/journals/corr/abs-1804-03209.bib},
  eprint = {1804.03209},
  eprinttype = {arXiv},
  journal = {CoRR},
  timestamp = {Mon, 13 Aug 2018 16:48:32 +0200},
  title = {Speech Commands: {A} Dataset for Limited-Vocabulary Speech Recognition},
  url = {http://arxiv.org/abs/1804.03209},
  volume = {abs/1804.03209},
  year = {2018},
}
""",
    )
    input_column_name: str = "audio"
    label_column_name: str = "label"

    def get_candidate_labels(self) -> list[str]:
        """Return the text candidates for zeroshot classification"""
        return [
            "Yes",
            "No",
            "Up",
            "Down",
            "Left",
            "Right",
            "On",
            "Off",
            "Stop",
            "Go",
            # Dataset has 30 labels, but only first 10 are used for zeroshot classification since they are considered as commands, others are considered as auxiliary labels for v1.1
        ]


class SpeechCommandsZeroshotClassificationv02(AbsTaskZeroShotClassification):
    metadata = TaskMetadata(
        name="SpeechCommandsZeroshotv0.02",
        description="Sound Classification/Keyword Spotting Dataset. This is a set of one-second audio clips containing a single spoken English word or background noise. These words are from a small set of commands such as 'yes', 'no', and 'stop' spoken by various speakers. With a total of 10 labels/commands for keyword spotting and a total of 30 labels for other auxiliary tasks",
        reference="https://huggingface.co/datasets/google/speech_commands",
        dataset={
            "path": "mteb/SpeechCommandsZeroshotv0.02",
            "revision": "d5491b288ab76356a5f3cbc634f1dc38e23da2b8",
        },
        type="AudioZeroshotClassification",
        category="a2t",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2018-07-07", "2018-07-13"),
        domains=["Spoken"],
        task_subtypes=["Keyword Spotting"],
        license="cc-by-4.0",  # Replace with appropriate license from allowed list
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio", "text"],
        sample_creation="found",
        bibtex_citation=r"""
@article{DBLP:journals/corr/abs-1804-03209,
  author = {Pete Warden},
  bibsource = {dblp computer science bibliography, https://dblp.org},
  biburl = {https://dblp.org/rec/journals/corr/abs-1804-03209.bib},
  eprint = {1804.03209},
  eprinttype = {arXiv},
  journal = {CoRR},
  timestamp = {Mon, 13 Aug 2018 16:48:32 +0200},
  title = {Speech Commands: {A} Dataset for Limited-Vocabulary Speech Recognition},
  url = {http://arxiv.org/abs/1804.03209},
  volume = {abs/1804.03209},
  year = {2018},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    def get_candidate_labels(self) -> list[str]:
        """Return the text candidates for zeroshot classification"""
        return [
            "Yes",
            "No",
            "Up",
            "Down",
            "Left",
            "Right",
            "On",
            "Off",
            "Stop",
            "Go",
            # Dataset has 30 labels, but only first 10 are used for zeroshot classification since they are considered as commands, others are considered as auxiliary labels for v1.1
        ]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class KoStrategyQA(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="Ko-StrategyQA",
        description="Ko-StrategyQA",
        reference="https://arxiv.org/abs/2101.02235",
        dataset={
            "path": "taeminlee/Ko-StrategyQA",
            "revision": "d243889a3eb6654029dbd7e7f9319ae31d58f97c",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["dev"],
        eval_langs=["kor-Hang"],
        main_score="ndcg_at_10",
        date=("2021-01-01", "2021-12-31"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@article{geva2021strategyqa,
  author = {Geva, Mor and Khashabi, Daniel and Segal, Elad and Khot, Tushar and Roth, Dan and Berant, Jonathan},
  journal = {Transactions of the Association for Computational Linguistics (TACL)},
  title = {{Did Aristotle Use a Laptop? A Question Answering Benchmark with Implicit Reasoning Strategies}},
  year = {2021},
}
""",
    )



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any

from mteb.abstasks.abstask import _set_seed

if TYPE_CHECKING:
    from collections.abc import Iterable, Mapping

    from mteb.models import EncoderProtocol
    from mteb.types import EncodeKwargs


class Evaluator(ABC):
    """Base class for all evaluators

    Extend this class and implement __call__ for custom evaluators.
    """

    def __init__(self, seed: int = 42, **kwargs: Any) -> None:
        self.seed = seed
        self.rng_state, self.np_rng = _set_seed(seed)

    @abstractmethod
    def __call__(
        self,
        model: EncoderProtocol,
        *,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None = None,
    ) -> Mapping[str, float] | Iterable[Any]:
        """This is called during training to evaluate the model.

        It returns scores.

        Args:
            model: the model to evaluate
            encode_kwargs: kwargs to pass to the model's encode method
            num_proc: number of processes to use for data loading
        """
        pass



from __future__ import annotations

import itertools
import logging
import random
from collections import defaultdict
from typing import TYPE_CHECKING, Any, cast

import numpy as np
from datasets import Dataset, DatasetDict
from sklearn.cluster import MiniBatchKMeans
from sklearn.metrics.cluster import v_measure_score

from mteb._create_dataloaders import create_dataloader
from mteb.models import EncoderProtocol
from mteb.types import Array, HFSubset
from mteb.types.statistics import (
    SplitDescriptiveStatistics,
)

from ._statistics_calculation import (
    calculate_audio_statistics,
    calculate_image_statistics,
    calculate_label_statistics,
    calculate_text_statistics,
)
from .abstask import AbsTask

if TYPE_CHECKING:
    from pathlib import Path

    from mteb.models import MTEBModels
    from mteb.types import Array, EncodeKwargs, ScoresDict
    from mteb.types.statistics import (
        AudioStatistics,
        ImageStatistics,
        LabelStatistics,
        TextStatistics,
    )

logger = logging.getLogger(__name__)


MultilingualDataset = dict[HFSubset, DatasetDict]


def _evaluate_clustering_bootstrapped(
    embeddings: Array,
    labels: list[list[str]],
    n_clusters: int,
    cluster_size: int,
    kmean_batch_size: int,
    max_depth: int | None,
    rng_state: random.Random,
    seed: int,
) -> tuple[dict[str, list[float]], dict[str, list[list[int]]]]:
    """Bootstrapped evaluation of clustering performance using V-measure.

    The bootstrapping is done by sampling N samples from the corpus and clustering them. It is done without replacement to get a diverse set of
    samples.

    Returns:
        A tuple containing:
        - A dictionary where keys are level names (e.g., "Level 0", "Level 1", etc.) and values are lists of V-measure scores for each clustering experiment at that level.
        - A dictionary where keys are level names and values are lists of cluster assignments for each clustering experiment at that level.
    """
    v_measures = defaultdict(list)
    cluster_assignments = defaultdict(list)
    if max_depth is not None:
        max_depth = min(max_depth, max(map(len, labels)))
    else:
        max_depth = max(map(len, labels))
    # Evaluate on each level til max depth
    for i_level in range(max_depth):
        level_labels: list[str | int] = []
        # Assign -1 to gold label if the level is not there
        for label in labels:
            if len(label) > i_level:
                level_labels.append(label[i_level])
            else:
                level_labels.append(-1)
        np_level_labels = np.array(level_labels)
        valid_idx = np.array(
            [level_label != -1 for level_label in np_level_labels]
        )  # Could be level_labels != -1 but fails with FutureWarning: elementwise comparison failed
        np_level_labels = np_level_labels[valid_idx]
        level_embeddings = embeddings[valid_idx]
        clustering_model = MiniBatchKMeans(
            n_clusters=np.unique(np_level_labels).size,
            batch_size=kmean_batch_size,
            init="k-means++",
            n_init=1,  # default when kmeans++ is used
            random_state=seed,
        )
        for _ in range(n_clusters):
            # sample N samples from the corpus with replacement
            n_embeddings = len(level_embeddings)
            cluster_indices = rng_state.choices(range(n_embeddings), k=cluster_size)

            _embeddings = level_embeddings[cluster_indices]
            _labels = np_level_labels[cluster_indices]
            cluster_assignment = clustering_model.fit_predict(_embeddings)
            v_measure = v_measure_score(_labels, cluster_assignment)
            v_measures[f"Level {i_level}"].append(v_measure)
            cluster_assignments[f"Level {i_level}"].append(cluster_assignment.tolist())

    return v_measures, cluster_assignments


class ClusteringFastDescriptiveStatistics(SplitDescriptiveStatistics):
    """Descriptive statistics for ClusteringFast

    Attributes:
        num_samples: number of samples in the dataset.

        text_statistics: Statistics for text
        image_statistics: Statistics for images
        audio_statistics: Statistics for audio
        labels_statistics: Statistics for labels
    """

    num_samples: int

    text_statistics: TextStatistics | None
    image_statistics: ImageStatistics | None
    audio_statistics: AudioStatistics | None
    labels_statistics: LabelStatistics


class AbsTaskClustering(AbsTask):
    """The abstract class for clustering tasks.

    A clustering task consists of a dataset with input data and corresponding cluster labels. The task is to cluster the input data points and compare
    the cluster assignments with the true cluster labels using V-measure.
    This class embeds the corpus sentences then samples N samples from the corpus and clusters them. The similarity then is calculated using multiple
    measures, including a V-measure. This approach is then repeated K times.

    If the clustering is hierarchical and more than one label is specified in order for each observation, we compute the metrics as outlined
    for each level separately.

    Attributes:
        dataset: A HuggingFace Dataset containing the data for the clustering task. Must contain the following columns `sentences` that contains
            inputs (texts or images) and labels columns.
        max_fraction_of_documents_to_embed: Fraction of documents to embed for clustering. Cannot be set at the same time as `max_document_to_embed`.
            If both are set to None, the entire dataset will be embedded for clustering.
        max_document_to_embed: Maximum number of documents to embed for clustering. Cannot be set at the same time as `max_fraction_of_documents_to_embed`.
            If both are set to None, the entire dataset will be embedded for clustering.
        max_documents_per_cluster: Number of documents to sample for each clustering experiment.
        n_clusters: Number of clustering experiments to run.
        k_mean_batch_size: Batch size to use for k-means clustering.
        max_depth: Maximum depth to evaluate clustering. If None, evaluates all levels.
        input_column_name: Name of the column containing the input sentences or data points.
        label_column_name: Name of the column containing the true cluster labels.
        abstask_prompt: Prompt to use for the task for instruction model if not prompt is provided in TaskMetadata.prompt.
    """

    max_fraction_of_documents_to_embed: float | None = 0.04
    max_document_to_embed: int | None = None
    max_documents_per_cluster: int = 16_384
    n_clusters: int = 10
    k_mean_batch_size: int = 512
    max_depth = None
    abstask_prompt = "Identify categories in user passages."
    input_column_name: str = "sentences"
    label_column_name: str = "labels"

    def _evaluate_subset(
        self,
        model: MTEBModels,
        data_split: Dataset,
        *,
        encode_kwargs: EncodeKwargs,
        hf_split: str,
        hf_subset: str,
        prediction_folder: Path | None = None,
        num_proc: int | None = None,
        **kwargs: Any,
    ) -> ScoresDict:
        if not isinstance(model, EncoderProtocol):
            raise TypeError(
                "Expected encoder model to be an instance of EncoderProtocol."
            )
        if (
            self.max_document_to_embed is not None
            and self.max_fraction_of_documents_to_embed is not None
        ):
            raise Exception(
                "Both max_document_to_embed and max_fraction_of_documents_to_embed are set. Please only set one."
            )

        logger.info("Running clustering - Preparing data...")
        if (
            self.max_document_to_embed is None
            and self.max_fraction_of_documents_to_embed is None
        ):
            downsampled_dataset = data_split
        else:
            if self.max_fraction_of_documents_to_embed is not None:
                max_documents_to_embed = int(
                    self.max_fraction_of_documents_to_embed * len(data_split)
                )
            else:
                max_documents_to_embed = cast("int", self.max_document_to_embed)

            max_documents_to_embed = min(len(data_split), max_documents_to_embed)
            example_indices = self.rng_state.sample(
                range(len(data_split)), k=max_documents_to_embed
            )
            downsampled_dataset = data_split.select(example_indices)

        downsampled_dataset = downsampled_dataset.select_columns(
            [self.input_column_name, self.label_column_name]
        )

        logger.info("Running clustering - Encoding samples...")
        embeddings = model.encode(
            create_dataloader(
                downsampled_dataset,
                self.metadata,
                input_column=self.input_column_name,
                num_proc=num_proc,
                **encode_kwargs,
            ),
            task_metadata=self.metadata,
            hf_subset=hf_subset,
            hf_split=hf_split,
            **encode_kwargs,
        )

        logger.info("Running clustering - Evaluating clustering...")
        labels = []
        for label in downsampled_dataset[self.label_column_name]:
            if not isinstance(label, list):
                label = [label]
            labels.append(label)

        all_v_scores, all_assignments = _evaluate_clustering_bootstrapped(
            embeddings,
            labels,
            n_clusters=self.n_clusters,
            cluster_size=self.max_documents_per_cluster,
            kmean_batch_size=self.k_mean_batch_size,
            max_depth=self.max_depth,
            rng_state=self.rng_state,
            seed=self.seed,
        )

        if prediction_folder:
            self._save_task_predictions(
                all_assignments,
                model,
                prediction_folder,
                hf_subset=hf_subset,
                hf_split=hf_split,
            )

        v_measures = list(itertools.chain.from_iterable(all_v_scores.values()))

        logger.info("Running clustering - Finished.")
        mean_v_measure = np.mean(v_measures)
        v_std = np.std(v_measures)
        return {
            "v_measures": all_v_scores,
            "v_measure": float(mean_v_measure),
            "v_measure_std": v_std,
        }

    def _calculate_descriptive_statistics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> ClusteringFastDescriptiveStatistics:
        if hf_subset:
            inputs = self.dataset[hf_subset][split][self.input_column_name]
            labels = self.dataset[hf_subset][split][self.label_column_name]
        elif compute_overall:
            inputs = []
            labels = []
            for hf_subset in self.metadata.eval_langs:
                inputs.extend(self.dataset[hf_subset][split][self.input_column_name])
                labels.extend(self.dataset[hf_subset][split][self.label_column_name])
        else:
            inputs = self.dataset[split][self.input_column_name]
            labels = self.dataset[split][self.label_column_name]

        if isinstance(inputs[0], list):
            inputs = [item for sublist in inputs for item in sublist]
        if isinstance(labels[0], list):
            labels = [item for sublist in labels for item in sublist]

        text_statistics, image_statistics, audio_statistics = None, None, None
        if "image" in self.metadata.modalities:
            image_statistics = calculate_image_statistics(inputs)

        if "text" in self.metadata.modalities:
            text_statistics = calculate_text_statistics(inputs)
        if "audio" in self.metadata.modalities:
            audio_statistics = calculate_audio_statistics(inputs)

        label_statistics = calculate_label_statistics(labels)

        return ClusteringFastDescriptiveStatistics(
            num_samples=len(inputs),
            text_statistics=text_statistics,
            image_statistics=image_statistics,
            audio_statistics=audio_statistics,
            labels_statistics=label_statistics,
        )

    def _push_dataset_to_hub(
        self,
        repo_name: str,
        num_proc: int | None = None,
    ) -> None:
        self._upload_dataset_to_hub(
            repo_name,
            [self.input_column_name, self.label_column_name],
            num_proc=num_proc,
        )


def _convert_to_fast(
    dataset: DatasetDict,
    input_column_name: str,
    label_column_name: str,
    seed: int,
    max_size: int = 100_000,
) -> DatasetDict:
    """Converts a clustering dataset to a fast version.

    This concat the cluster into two columns, sentences and labels. It additionally downsamples the dataset to max_size.

    Args:
        dataset: A DatasetDict containing the data for the clustering task. Must contain the following columns
        input_column_name: Name of the column containing the input sentences or data points.
        label_column_name: Name of the column containing the true cluster labels.
        seed: Random seed for downsampling.
        max_size: Maximum number of samples in the returned dataset.

    Returns:
        A downsampled DatasetDict with two columns, sentences and labels.
    """
    rng_state = random.Random(seed)

    ds = {}
    for split in dataset:
        sent_set = set()
        labels = []
        sentences = []
        n_clusters = len(dataset[split])
        all_labels_set = set(
            itertools.chain.from_iterable(dataset[split][label_column_name])
        )
        for i in range(n_clusters):
            lab = dataset[split][label_column_name][i]
            sents = dataset[split][input_column_name][i]

            # check that it is the same distribution
            row_label_set = set(lab)
            assert row_label_set.issubset(all_labels_set), (
                "The clusters are not sampled from the same distribution as they have different labels."
            )

            for l, s in zip(lab, sents):
                if s not in sent_set:
                    labels.append(l)
                    sentences.append(s)
                    sent_set.add(s)  # ensuring no duplicates

        ds[split] = Dataset.from_dict(
            {input_column_name: sentences, label_column_name: labels}
        )

        if len(ds[split]) > max_size:
            idxs = rng_state.sample(range(len(ds[split])), max_size)
            ds[split] = ds[split].select(idxs)

    return DatasetDict(ds)


def _check_label_distribution(
    ds: DatasetDict,
    label_column_name: str = "labels",
) -> None:
    """For older clustering dataset versions.

    Checks that all clusters are sampled from the same distribution by checking that the set of labels is the same across clusters.

    Args:
        ds: A DatasetDict containing the data for the clustering task. Must contain the following columns
        label_column_name: Name of the column containing the true cluster labels.
    """
    n_clusters = len(ds)
    if n_clusters > 50:
        return
    all_labels_set = set(itertools.chain.from_iterable(ds[label_column_name]))

    for i in range(n_clusters):
        lab = ds[label_column_name][i]

        # check that it is the same distribution
        row_label_set = set(lab)
        assert row_label_set.issubset(all_labels_set), (
            "The clusters are not sampled from the same distribution as they have different labels."
        )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CzechSubjectivityClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="CzechSubjectivityClassification",
        description="An Czech dataset for subjectivity classification.",
        reference="https://arxiv.org/abs/2009.08712",
        dataset={
            "path": "pauli31/czech-subjectivity-dataset",
            "revision": "e387ddf167f3eba99936cff89909ed6264f17e1f",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        date=("2022-04-01", "2022-04-01"),
        eval_splits=["validation", "test"],
        eval_langs=["ces-Latn"],
        main_score="accuracy",
        domains=["Reviews", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{priban-steinberger-2022-czech,
  address = {Marseille, France},
  author = {P{\v{r}}ib{\'a}{\v{n}}, Pavel  and
Steinberger, Josef},
  booktitle = {Proceedings of the Thirteenth Language Resources and Evaluation Conference},
  month = jun,
  pages = {1381--1391},
  publisher = {European Language Resources Association},
  title = {\{C\}zech Dataset for Cross-lingual Subjectivity Classification},
  url = {https://aclanthology.org/2022.lrec-1.148},
  year = {2022},
}
""",
    )



from .georeview_classification import GeoreviewClassification, GeoreviewClassificationV2
from .headline_classification import HeadlineClassification, HeadlineClassificationV2
from .inappropriateness_classification import (
    InappropriatenessClassification,
    InappropriatenessClassificationV2,
    InappropriatenessClassificationv2,
)
from .kinopoisk_classification import KinopoiskClassification
from .ru_reviews_classification import (
    RuReviewsClassification,
    RuReviewsClassificationV2,
)
from .ru_sci_bench_grnti_classification import RuSciBenchGRNTIClassification
from .ru_sci_bench_oecd_classification import RuSciBenchOECDClassification
from .ru_toixic_classification_okmlcup import (
    RuToxicOKMLCUPClassification,
    RuToxicOKMLCUPClassificationV2,
)
from .senti_ru_eval import (
    SentiRuEval2016Classification,
    SentiRuEval2016ClassificationV2,
)

__all__ = [
    "GeoreviewClassification",
    "GeoreviewClassificationV2",
    "HeadlineClassification",
    "HeadlineClassificationV2",
    "InappropriatenessClassification",
    "InappropriatenessClassificationV2",
    "InappropriatenessClassificationv2",
    "KinopoiskClassification",
    "RuReviewsClassification",
    "RuReviewsClassificationV2",
    "RuSciBenchGRNTIClassification",
    "RuSciBenchOECDClassification",
    "RuToxicOKMLCUPClassification",
    "RuToxicOKMLCUPClassificationV2",
    "SentiRuEval2016Classification",
    "SentiRuEval2016ClassificationV2",
]



from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class OpenTenderClusteringP2P(AbsTaskClustering):
    max_fraction_of_documents_to_embed = 1.0
    metadata = TaskMetadata(
        name="OpenTenderClusteringP2P",
        dataset={
            "path": "clips/mteb-nl-opentender-cls-pr",
            "revision": "9af5657575a669dc18c7f897a67287ff7d1a0c65",
        },
        description="This dataset contains all the articles published by the NOS as of the 1st of January 2010. The "
        "data is obtained by scraping the NOS website. The NOS is one of the biggest (online) news "
        "organizations in the Netherlands.",
        reference="https://arxiv.org/abs/2509.12340",
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="v_measure",
        date=("2025-08-01", "2025-08-10"),
        domains=["Government", "Written"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{banar2025mtebnle5nlembeddingbenchmark,
  archiveprefix = {arXiv},
  author = {Nikolay Banar and Ehsan Lotfi and Jens Van Nooten and Cristina Arhiliuc and Marija Kliocaite and Walter Daelemans},
  eprint = {2509.12340},
  primaryclass = {cs.CL},
  title = {MTEB-NL and E5-NL: Embedding Benchmark and Models for Dutch},
  url = {https://arxiv.org/abs/2509.12340},
  year = {2025},
}
""",
        prompt="Identificeer de hoofdcategorie van aanbestedingen op basis van de titels en beschrijvingen",
    )

    def dataset_transform(
        self,
        num_proc: int | None = None,
    ):
        # reuse the dataset for classification
        for split in self.dataset:
            self.dataset[split] = self.dataset[split].map(
                lambda ex: {
                    "labels": ex["label"],
                    "sentences": ex["text"],
                }
            )



from .livedoor_news_clustering import LivedoorNewsClustering, LivedoorNewsClusteringv2
from .mews_c16_ja_clustering import MewsC16JaClustering

__all__ = ["LivedoorNewsClustering", "LivedoorNewsClusteringv2", "MewsC16JaClustering"]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

DOMAINS = [
    "academic_paper",
    "admin_industry",
    "brochure",
    "financial_report",
    "government",
    "guidebook",
    "laws",
    "news",
    "research_report",
    "tutorial_workshop",
]


class MMDocIRT2ITRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="MMDocIRT2ITRetrieval",
        description="MMDocIR evaluation set includes 313 long documents averaging 65.1 pages, categorized into ten main domains: research reports, administration&industry, tutorials&workshops, academic papers, brochures, financial reports, guidebooks, government documents, laws, and news articles. Different domains feature distinct distributions of multi-modal information. Overall, the modality distribution is: Text (60.4%), Image (18.8%), Table (16.7%), and other modalities (4.1%).",
        reference="https://arxiv.org/abs/2501.08828",
        dataset={
            "path": "mteb/MMDocIRT2ITRetrieval",
            "revision": "ef97f91ffcd80abf10dda2f72f5971c18c5d5e74",
        },
        type="DocumentUnderstanding",
        category="t2it",
        eval_splits=["train"],
        eval_langs={domain: ["eng-Latn"] for domain in DOMAINS},
        main_score="recall_at_5",
        date=("2025-01-01", "2025-01-01"),
        domains=["Academic", "Non-fiction", "Government", "Legal", "News"],
        task_subtypes=["Image Text Retrieval"],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        modalities=["text", "image"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{dong2025mmdocirbenchmarkingmultimodalretrieval,
  archiveprefix = {arXiv},
  author = {Kuicai Dong and Yujing Chang and Xin Deik Goh and Dexun Li and Ruiming Tang and Yong Liu},
  eprint = {2501.08828},
  primaryclass = {cs.IR},
  title = {MMDocIR: Benchmarking Multi-Modal Retrieval for Long Documents},
  url = {https://arxiv.org/abs/2501.08828},
  year = {2025},
}
""",
    )



from mteb.abstasks.clustering import AbsTaskClustering
from mteb.abstasks.clustering_legacy import AbsTaskClusteringLegacy
from mteb.abstasks.task_metadata import TaskMetadata


class SwednClustering(AbsTaskClusteringLegacy):
    metadata = TaskMetadata(
        name="SwednClustering",
        dataset={
            "path": "mteb/SwednClustering",
            "revision": "45d2a99c3f1b6ee6189a6bb762ed74b7ef45dd9d",
        },
        description="The SWE-DN corpus is based on 1,963,576 news articles from the Swedish newspaper Dagens Nyheter (DN) during the years 2000--2020. The articles are filtered to resemble the CNN/DailyMail dataset both regarding textual structure. This dataset uses the category labels as clusters.",
        reference="https://spraakbanken.gu.se/en/resources/swedn",
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="v_measure",
        date=("2000-01-01", "2020-12-31"),  # best guess
        domains=["News", "Non-fiction", "Written"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        task_subtypes=["Thematic clustering"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{monsen2021method,
  author = {Monsen, Julius and J{\"o}nsson, Arne},
  booktitle = {Proceedings of CLARIN Annual Conference},
  title = {A method for building non-english corpora for abstractive text summarization},
  year = {2021},
}
""",
        superseded_by="SwednClusteringP2P",
    )


class SwednClusteringP2P(AbsTaskClustering):
    max_document_to_embed = 2048
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="SwednClusteringP2P",
        dataset={
            "path": "mteb/SwednClusteringP2P",
            "revision": "f8dbf10ec231cc25e9f63454d5cd2d90af95e5f8",
        },
        description="The SWE-DN corpus is based on 1,963,576 news articles from the Swedish newspaper Dagens Nyheter (DN) during the years 2000--2020. The articles are filtered to resemble the CNN/DailyMail dataset both regarding textual structure. This dataset uses the category labels as clusters.",
        reference="https://spraakbanken.gu.se/en/resources/swedn",
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["summaries", "articles"],
        eval_langs=["swe-Latn"],
        main_score="v_measure",
        date=("2000-01-01", "2020-12-31"),  # best guess
        domains=["News", "Non-fiction", "Written"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        task_subtypes=["Thematic clustering"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{monsen2021method,
  author = {Monsen, Julius and J{\"o}nsson, Arne},
  booktitle = {Proceedings of CLARIN Annual Conference},
  title = {A method for building non-english corpora for abstractive text summarization},
  year = {2021},
}
""",
        prompt="Identify news categories in Swedish passages",
    )


class SwednClusteringFastS2S(AbsTaskClustering):
    max_document_to_embed = 2048
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="SwednClusteringS2S",
        dataset={
            "path": "mteb/SwednClusteringS2S",
            "revision": "4dc1f92a8d5c4fe4be7995baa8009384f46d98d6",
        },
        description="The SWE-DN corpus is based on 1,963,576 news articles from the Swedish newspaper Dagens Nyheter (DN) during the years 2000--2020. The articles are filtered to resemble the CNN/DailyMail dataset both regarding textual structure. This dataset uses the category labels as clusters.",
        reference="https://spraakbanken.gu.se/en/resources/swedn",
        type="Clustering",
        category="t2c",
        modalities=["text"],
        eval_splits=["headlines"],
        eval_langs=["swe-Latn"],
        main_score="v_measure",
        date=("2000-01-01", "2020-12-31"),  # best guess
        domains=["News", "Non-fiction", "Written"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=[],
        task_subtypes=["Thematic clustering"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{monsen2021method,
  author = {Monsen, Julius and J{\"o}nsson, Arne},
  booktitle = {Proceedings of CLARIN Annual Conference},
  title = {A method for building non-english corpora for abstractive text summarization},
  year = {2021},
}
""",
        prompt="Identify news categories in Swedish passages",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class FrenkEnClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FrenkEnClassification",
        description="English subset of the FRENK dataset",
        dataset={
            "path": "mteb/FrenkEnClassification",
            "revision": "d7abc39711be5ec3d803826506eb6137108e66ed",
        },
        reference="https://arxiv.org/abs/1906.02045",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2021-05-28", "2021-05-28"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{ljubešić2019frenk,
  archiveprefix = {arXiv},
  author = {Nikola Ljubešić and Darja Fišer and Tomaž Erjavec},
  eprint = {1906.02045},
  primaryclass = {cs.CL},
  title = {The FRENK Datasets of Socially Unacceptable Discourse in Slovene and English},
  url = {https://arxiv.org/abs/1906.02045},
  year = {2019},
}
""",
        superseded_by="FrenkEnClassification.v2",
    )


class FrenkEnClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FrenkEnClassification.v2",
        description="English subset of the FRENK dataset This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        dataset={
            "path": "mteb/frenk_en",
            "revision": "630d941b6e0879a7238da89af6bfe1b1eb27ca0f",
        },
        reference="https://arxiv.org/abs/1906.02045",
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2021-05-28", "2021-05-28"),
        domains=["Social", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{ljubešić2019frenk,
  archiveprefix = {arXiv},
  author = {Nikola Ljubešić and Darja Fišer and Tomaž Erjavec},
  eprint = {1906.02045},
  primaryclass = {cs.CL},
  title = {The FRENK Datasets of Socially Unacceptable Discourse in Slovene and English},
  url = {https://arxiv.org/abs/1906.02045},
  year = {2019},
}
""",
        adapted_from=["FrenkEnClassification"],
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import torch
from tqdm.auto import tqdm
from transformers import Wav2Vec2FeatureExtractor, Wav2Vec2ForCTC, Wav2Vec2Model

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_audio_dependencies
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


# ISO 639-3 codes for languages supported by wav2vec2 models
WAV2VEC2_LANGUAGES = [
    "afr-Latn",
    "sqi-Latn",
    "amh-Latn",
    "ara-Latn",
    "hye-Latn",
    "asm-Latn",
    "aze-Latn",
    "eus-Latn",
    "bel-Latn",
    "ben-Beng",
    "bos-Latn",
    "bre-Latn",
    "bul-Latn",
    "mya-Latn",
    "cat-Latn",
    "khm-Latn",
    "zho-Latn",
    "hrv-Latn",
    "ces-Latn",
    "dan-Latn",
    "nld-Latn",
    "eng-Latn",
    "epo-Latn",
    "est-Latn",
    "fin-Latn",
    "fra-Latn",
    "glg-Latn",
    "kat-Latn",
    "deu-Latn",
    "ell-Latn",
    "guj-Latn",
    "hau-Latn",
    "heb-Latn",
    "hin-Deva",
    "hun-Latn",
    "isl-Latn",
    "ind-Latn",
    "gle-Latn",
    "ita-Latn",
    "jpn-Latn",
    "jav-Latn",
    "kan-Latn",
    "kaz-Latn",
    "kir-Latn",
    "abk-Cyrl",
    "bak-Cyrl",
    "ceb-Latn",
    "chv-Cyrl",
    "div-Thaa",
    "fao-Latn",
    "grn-Latn",
    "hat-Latn",
    "haw-Latn",
    "ina-Latn",
    "kin-Latn",
]


class Wav2Vec2AudioWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_seconds: float = 30.0,
        **kwargs: Any,
    ):
        requires_audio_dependencies()
        self.model_name = model_name
        self.device = device
        self.max_audio_length_seconds = max_audio_length_seconds

        # Try to load base model first, fallback to CTC if needed
        try:
            self.model = Wav2Vec2Model.from_pretrained(
                model_name, revision=revision
            ).to(self.device)
            self.is_ctc_model = False
        except Exception:
            # Fallback to CTC model for models that don't have base versions
            self.model = Wav2Vec2ForCTC.from_pretrained(
                model_name, revision=revision
            ).to(self.device)
            self.is_ctc_model = True

        self.model.eval()

        self.feature_extractor = Wav2Vec2FeatureExtractor.from_pretrained(model_name)
        self.sampling_rate = self.feature_extractor.sampling_rate

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        for batch in tqdm(
            inputs,
            disable=not show_progress_bar,
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            feature_inputs = self.feature_extractor(
                audio_arrays,
                sampling_rate=self.sampling_rate,
                return_tensors="pt",
                padding="longest",
                truncation=True,
                max_length=int(self.max_audio_length_seconds * self.sampling_rate),
                return_attention_mask=True,
            ).to(self.device)

            with torch.no_grad():
                outputs = self.model(
                    feature_inputs.input_values,
                    attention_mask=feature_inputs.attention_mask,
                    output_hidden_states=True,
                )

                last_hidden_state = (
                    outputs.hidden_states[-1]
                    if self.is_ctc_model
                    else outputs.last_hidden_state
                )

                # last_hidden_state: [B, hidden_seq_len, hidden_size]
                batch_size, hidden_seq_len, hidden_size = last_hidden_state.shape
                device = last_hidden_state.device

                # inputs.attention_mask is per-sample mask over input_values
                input_lengths = (
                    feature_inputs.attention_mask.sum(dim=1)
                    .cpu()
                    .detach()
                    .numpy()
                    .astype(int)
                )  # shape (B,)

                hidden_lengths = [
                    self.model._get_feat_extract_output_lengths(l)
                    for l in input_lengths
                ]
                hidden_lengths = torch.tensor(
                    hidden_lengths, device=device, dtype=torch.long
                )  # (B,)

                # clamp to be safe
                hidden_lengths = torch.clamp(hidden_lengths, min=0, max=hidden_seq_len)

                # build mask vectorized: seq_range shape [1, hidden_seq_len]
                seq_range = torch.arange(hidden_seq_len, device=device).unsqueeze(
                    0
                )  # [1, hidden_seq_len]
                hidden_attention_mask = (seq_range < hidden_lengths.unsqueeze(1)).to(
                    last_hidden_state.dtype
                )  # [B, hidden_seq_len]

                # apply masked mean pooling
                hidden_attention_mask = hidden_attention_mask.unsqueeze(
                    -1
                )  # [B, hidden_seq_len, 1]
                masked_embeddings = (
                    last_hidden_state * hidden_attention_mask
                )  # [B, hidden_seq_len, hidden_size]
                valid_tokens = hidden_attention_mask.sum(dim=1)  # [B, 1]

                # Safer division to avoid NaN
                sum_embeddings = masked_embeddings.sum(dim=1)  # [B, hidden_size]

                # Check for NaN in the sum and replace with zeros
                nan_mask = torch.isnan(sum_embeddings).any(
                    dim=1, keepdim=True
                )  # [B, 1]
                sum_embeddings = torch.where(
                    nan_mask.expand_as(sum_embeddings),
                    torch.zeros_like(sum_embeddings),
                    sum_embeddings,
                )

                valid_tokens_safe = torch.where(
                    valid_tokens > 0, valid_tokens, torch.ones_like(valid_tokens)
                )
                embeddings = sum_embeddings / valid_tokens_safe  # [B, hidden_size]

                # Zero out embeddings where we had no valid tokens
                zero_mask = (valid_tokens <= 0).expand_as(embeddings)
                embeddings = torch.where(
                    zero_mask, torch.zeros_like(embeddings), embeddings
                )

                # Final NaN check and replacement
                embeddings = torch.where(
                    torch.isnan(embeddings), torch.zeros_like(embeddings), embeddings
                )

                all_embeddings.append(embeddings.cpu().detach())

        return torch.cat(all_embeddings, dim=0).numpy()

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        if "audio" not in inputs.dataset.features:
            raise ValueError("Wav2Vec2Wrapper only supports audio inputs.")
        return self.get_audio_embeddings(inputs, **kwargs)


wav2vec2_xlsr_300m = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-xls-r-300m",
    languages=WAV2VEC2_LANGUAGES,
    revision="1a640f32ac3e39899438a2931f9924c02f080a54",
    release_date="2021-10-13",
    modalities=["audio"],
    n_parameters=300_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=1200,
    max_tokens=float("inf"),
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/pytorch/fairseq",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/wav2vec2-xls-r-300m",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=set(),
    citation="""
@misc{babu2021xlsrselfsupervisedcrosslingualspeech,
      title={XLS-R: Self-supervised Cross-lingual Speech Representation Learning at Scale},
      author={Arun Babu and Changhan Wang and Andros Tjandra and Kushal Lakhotia and Qiantong Xu and Naman Goyal and Kritika Singh and Patrick von Platen and Yatharth Saraf and Juan Pino and Alexei Baevski and Alexis Conneau and Michael Auli},
      year={2021},
      eprint={2111.09296},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2111.09296},
}""",
)

wav2vec2_xlsr_300m_phoneme = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="vitouphy/wav2vec2-xls-r-300m-phoneme",
    languages=["eng-Latn"],
    revision="bf9913bf096d133cf4eca64ed75981ebf0545c9d",
    release_date="2022-05-19",
    modalities=["audio"],
    n_parameters=300_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=1200,
    max_tokens=float("inf"),
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/vitouphy/wav2vec2-xls-r-300m-phoneme",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=None,
    citation="""
@misc{babu2021xlsrselfsupervisedcrosslingualspeech,
      title={XLS-R: Self-supervised Cross-lingual Speech Representation Learning at Scale},
      author={Arun Babu and Changhan Wang and Andros Tjandra and Kushal Lakhotia and Qiantong Xu and Naman Goyal and Kritika Singh and Patrick von Platen and Yatharth Saraf and Juan Pino and Alexei Baevski and Alexis Conneau and Michael Auli},
      year={2021},
      eprint={2111.09296},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2111.09296},
}""",
)

wav2vec2_xlsr_1b = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-xls-r-1b",
    languages=WAV2VEC2_LANGUAGES,
    revision="35eaea9a0ed0f97592277d79208e40ab8917d1e3",
    release_date="2024-09-10",
    modalities=["audio"],
    n_parameters=1_000_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=4500,
    max_tokens=float("inf"),
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/pytorch/fairseq",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/wav2vec2-xls-r-1b",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=None,
    citation="""
@misc{babu2021xlsrselfsupervisedcrosslingualspeech,
      title={XLS-R: Self-supervised Cross-lingual Speech Representation Learning at Scale},
      author={Arun Babu and Changhan Wang and Andros Tjandra and Kushal Lakhotia and Qiantong Xu and Naman Goyal and Kritika Singh and Patrick von Platen and Yatharth Saraf and Juan Pino and Alexei Baevski and Alexis Conneau and Michael Auli},
      year={2021},
      eprint={2111.09296},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2111.09296},
}""",
)

wav2vec2_xlsr_2b = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-xls-r-2b",
    languages=WAV2VEC2_LANGUAGES,
    revision="3b6d89d0fabead7da552eaaa07549c7c9c36d303",
    release_date="2024-09-10",
    modalities=["audio"],
    n_parameters=2_000_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=9000,
    max_tokens=float("inf"),
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code="https://github.com/pytorch/fairseq",
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/wav2vec2-xls-r-2b",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=None,
    citation="""
@misc{babu2021xlsrselfsupervisedcrosslingualspeech,
      title={XLS-R: Self-supervised Cross-lingual Speech Representation Learning at Scale},
      author={Arun Babu and Changhan Wang and Andros Tjandra and Kushal Lakhotia and Qiantong Xu and Naman Goyal and Kritika Singh and Patrick von Platen and Yatharth Saraf and Juan Pino and Alexei Baevski and Alexis Conneau and Michael Auli},
      year={2021},
      eprint={2111.09296},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2111.09296},
}""",
)

wav2vec2_xlsr_2b_translation = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-xls-r-2b-21-to-en",
    languages=WAV2VEC2_LANGUAGES,
    revision="70239d15f5b39ecbc936a5e214bf401b7f17e210",
    release_date="2024-09-10",
    modalities=["audio"],
    n_parameters=2_000_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=9200,
    max_tokens=float("inf"),
    embed_dim=1024,
    license="apache-2.0",
    open_weights=True,
    public_training_code=None,
    public_training_data=None,
    framework=["PyTorch"],
    reference="https://huggingface.co/facebook/wav2vec2-xls-r-2b-21-to-en",
    similarity_fn_name="cosine",
    use_instructions=False,
    training_datasets=None,
    citation="""
@misc{babu2021xlsrselfsupervisedcrosslingualspeech,
      title={XLS-R: Self-supervised Cross-lingual Speech Representation Learning at Scale},
      author={Arun Babu and Changhan Wang and Andros Tjandra and Kushal Lakhotia and Qiantong Xu and Naman Goyal and Kritika Singh and Patrick von Platen and Yatharth Saraf and Juan Pino and Alexei Baevski and Alexis Conneau and Michael Auli},
      year={2021},
      eprint={2111.09296},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2111.09296},
}""",
)


wav2vec2_base = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-base",
    languages=["eng-Latn"],
    open_weights=True,
    revision="0b5b8e868dd84f03fd87d01f9c4ff0f080fecfe8",
    release_date="2020-10-26",
    max_tokens=float("inf"),
    n_parameters=95_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=362,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/facebook/wav2vec2-base",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{baevski2020wav2vec20frameworkselfsupervised,
      title={wav2vec 2.0: A Framework for Self-Supervised Learning of Speech Representations},
      author={Alexei Baevski and Henry Zhou and Abdelrahman Mohamed and Michael Auli},
      year={2020},
      eprint={2006.11477},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2006.11477},
}""",
)


wav2vec2_base_960h = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-base-960h",
    languages=["eng-Latn"],
    open_weights=True,
    revision="22aad52d435eb6dbaf354bdad9b0da84ce7d6156",
    release_date="2020-10-26",
    max_tokens=float("inf"),
    n_parameters=95_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=360,
    embed_dim=768,
    license="apache-2.0",
    reference="https://huggingface.co/facebook/wav2vec2-base-960h",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{baevski2020wav2vec20frameworkselfsupervised,
      title={wav2vec 2.0: A Framework for Self-Supervised Learning of Speech Representations},
      author={Alexei Baevski and Henry Zhou and Abdelrahman Mohamed and Michael Auli},
      year={2020},
      eprint={2006.11477},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2006.11477},
}""",
)


wav2vec2_large = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-large",
    languages=["eng-Latn"],
    open_weights=True,
    revision="312b2410566b698c7a649068d413b2067848bd75",
    release_date="2020-10-26",
    max_tokens=float("inf"),
    n_parameters=317_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=1_209,
    embed_dim=1_024,
    license="apache-2.0",
    reference="https://huggingface.co/facebook/wav2vec2-large",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{baevski2020wav2vec20frameworkselfsupervised,
      title={wav2vec 2.0: A Framework for Self-Supervised Learning of Speech Representations},
      author={Alexei Baevski and Henry Zhou and Abdelrahman Mohamed and Michael Auli},
      year={2020},
      eprint={2006.11477},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2006.11477},
}""",
)


wav2vec2_large_xlsr_53 = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-large-xlsr-53",
    languages=["eng-Latn"],
    open_weights=True,
    revision="c3f9d884181a224a6ac87bf8885c84d1cff3384f",
    release_date="2020-10-26",
    max_tokens=float("inf"),
    n_parameters=317_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=1_209,
    embed_dim=1_024,
    license="apache-2.0",
    reference="https://huggingface.co/facebook/wav2vec2-large-xlsr-53",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{conneau2020unsupervisedcrosslingualrepresentationlearning,
      title={Unsupervised Cross-lingual Representation Learning for Speech Recognition},
      author={Alexis Conneau and Alexei Baevski and Ronan Collobert and Abdelrahman Mohamed and Michael Auli},
      year={2020},
      eprint={2006.13979},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2006.13979},
}""",
)


wav2vec2_lv_60_espeak_cv_ft = ModelMeta(
    loader=Wav2Vec2AudioWrapper,
    name="facebook/wav2vec2-lv-60-espeak-cv-ft",
    languages=["eng-Latn"],
    open_weights=True,
    revision="ae45363bf3413b374fecd9dc8bc1df0e24c3b7f4",
    release_date="2020-10-26",
    max_tokens=float("inf"),
    n_parameters=317_000_000,
    n_embedding_parameters=0,
    memory_usage_mb=1_209,
    embed_dim=1_024,
    license="apache-2.0",
    reference="https://huggingface.co/facebook/wav2vec2-lv-60-espeak-cv-ft",
    similarity_fn_name="cosine",
    framework=["PyTorch"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    modalities=["audio"],
    citation="""
@misc{baevski2020wav2vec20frameworkselfsupervised,
      title={wav2vec 2.0: A Framework for Self-Supervised Learning of Speech Representations},
      author={Alexei Baevski and Henry Zhou and Abdelrahman Mohamed and Michael Auli},
      year={2020},
      eprint={2006.11477},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={https://arxiv.org/abs/2006.11477},
}""",
)



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class NoRecClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NoRecClassification",
        description="A Norwegian dataset for sentiment classification on review",
        reference="https://aclanthology.org/L18-1661/",
        dataset={
            # using the mini version to keep results ~comparable to the ScandEval benchmark
            "path": "mteb/norec_classification",
            "revision": "5b740b7c42c73d586420812a35745fc37118862f",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nob-Latn"],
        main_score="accuracy",
        date=("1998-01-01", "2018-01-01"),  # based on plot in paper
        domains=["Written", "Reviews"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{velldal-etal-2018-norec,
  address = {Miyazaki, Japan},
  author = {Velldal, Erik  and
{\\O}vrelid, Lilja  and
Bergem, Eivind Alexander  and
Stadsnes, Cathrine  and
Touileb, Samia  and
J{\\o}rgensen, Fredrik},
  booktitle = {Proceedings of the Eleventh International Conference on Language Resources and Evaluation ({LREC} 2018)},
  editor = {Calzolari, Nicoletta  and
Choukri, Khalid  and
Cieri, Christopher  and
Declerck, Thierry  and
Goggi, Sara  and
Hasida, Koiti  and
Isahara, Hitoshi  and
Maegaard, Bente  and
Mariani, Joseph  and
Mazo, H{\\'e}l{\\`e}ne  and
Moreno, Asuncion  and
Odijk, Jan  and
Piperidis, Stelios  and
Tokunaga, Takenobu},
  month = may,
  publisher = {European Language Resources Association (ELRA)},
  title = {{N}o{R}e{C}: The {N}orwegian Review Corpus},
  url = {https://aclanthology.org/L18-1661},
  year = {2018},
}
""",
        prompt="Classify Norwegian reviews by sentiment",
        superseded_by="NoRecClassification.v2",
    )


class NoRecClassificationV2(AbsTaskClassification):
    metadata = TaskMetadata(
        name="NoRecClassification.v2",
        description="A Norwegian dataset for sentiment classification on review This version corrects errors found in the original data. For details, see [pull request](https://github.com/embeddings-benchmark/mteb/pull/2900)",
        reference="https://aclanthology.org/L18-1661/",
        dataset={
            # using the mini version to keep results ~comparable to the ScandEval benchmark
            "path": "mteb/no_rec",
            "revision": "10aae1fb3fe2c19888bd4ea11695bbf19aa8bed3",
        },
        type="Classification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nob-Latn"],
        main_score="accuracy",
        date=("1998-01-01", "2018-01-01"),  # based on plot in paper
        domains=["Written", "Reviews"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{velldal-etal-2018-norec,
  address = {Miyazaki, Japan},
  author = {Velldal, Erik  and
{\\O}vrelid, Lilja  and
Bergem, Eivind Alexander  and
Stadsnes, Cathrine  and
Touileb, Samia  and
J{\\o}rgensen, Fredrik},
  booktitle = {Proceedings of the Eleventh International Conference on Language Resources and Evaluation ({LREC} 2018)},
  editor = {Calzolari, Nicoletta  and
Choukri, Khalid  and
Cieri, Christopher  and
Declerck, Thierry  and
Goggi, Sara  and
Hasida, Koiti  and
Isahara, Hitoshi  and
Maegaard, Bente  and
Mariani, Joseph  and
Mazo, H{\\'e}l{\\`e}ne  and
Moreno, Asuncion  and
Odijk, Jan  and
Piperidis, Stelios  and
Tokunaga, Takenobu},
  month = may,
  publisher = {European Language Resources Association (ELRA)},
  title = {{N}o{R}e{C}: The {N}orwegian Review Corpus},
  url = {https://aclanthology.org/L18-1661},
  year = {2018},
}
""",
        prompt="Classify Norwegian reviews by sentiment",
        adapted_from=["NoRecClassification"],
    )



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class CTKFactsNLI(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="CTKFactsNLI",
        dataset={
            "path": "mteb/CTKFactsNLI",
            "revision": "da834e176f18e4a4adc4cbc9ad9720c14168a4bb",
        },
        description="Czech Natural Language Inference dataset of around 3K evidence-claim pairs labelled with SUPPORTS, REFUTES or NOT ENOUGH INFO veracity labels. Extracted from a round of fact-checking experiments.",
        reference="https://arxiv.org/abs/2201.11115",
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["validation", "test"],
        eval_langs=["ces-Latn"],
        main_score="max_ap",
        date=("2020-09-01", "2021-08-31"),  # academic year 2020/2021
        domains=["News", "Written"],
        task_subtypes=["Claim verification"],
        license="cc-by-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{ullrich2023csfever,
  author = {Ullrich, Herbert and Drchal, Jan and R{\\`y}par, Martin and Vincourov{\\'a}, Hana and Moravec, V{\\'a}clav},
  journal = {Language Resources and Evaluation},
  number = {4},
  pages = {1571--1605},
  publisher = {Springer},
  title = {CsFEVER and CTKFacts: acquiring Czech data for fact verification},
  volume = {57},
  year = {2023},
}
""",  # after removing label 1=NOT ENOUGH INFO
    )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm
from transformers import CLIPModel, CLIPProcessor

from mteb._create_dataloaders import AudioCollator
from mteb._requires_package import requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput, TextInput


class Wav2ClipZeroShotWrapper(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        max_audio_length_s: float = 30.0,
        **kwargs: Any,
    ):
        requires_package(self, "wav2clip", "pip install 'mteb[wav2clip]'")
        from wav2clip import embed_audio, get_model

        self.embed_audio = embed_audio
        # audio side
        self.device = device
        self.audio_model = get_model().to(device)
        self.sampling_rate = 16_000
        self.max_audio_length_s = max_audio_length_s

        # text side (CLIP) - we use the standard OpenAI CLIP model as mentioned in paper
        # Wav2Clip aligns audio embeddings to CLIP's embedding space using this specific model
        clip_model_name = "openai/clip-vit-base-patch32"
        clip_revision = "3d74acf9a28c67741b2f4f2ea7635f0aaf6f0268"
        self.clip = CLIPModel.from_pretrained(
            clip_model_name, revision=clip_revision, device_map=device
        )
        self.clip.eval()
        self.clip_processor = CLIPProcessor.from_pretrained(
            clip_model_name, revision=clip_revision
        )

    def get_audio_embeddings(
        self,
        inputs: DataLoader[AudioInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> np.ndarray:
        inputs.collate_fn = AudioCollator(self.sampling_rate)

        all_embeddings = []

        # Process each DataLoader batch separately
        for batch in tqdm(
            inputs, desc="Processing audio batches", disable=not show_progress_bar
        ):
            audio_arrays = [audio["array"] for audio in batch["audio"]]

            max_length = max(wav.shape[-1] for wav in audio_arrays)
            padded_wavs = []
            for wav in audio_arrays:
                if wav.shape[-1] < max_length:
                    # Pad with zeros
                    pad_length = max_length - wav.shape[-1]
                    padded_wav = torch.nn.functional.pad(wav, (0, pad_length))
                else:
                    padded_wav = wav
                padded_wavs.append(padded_wav)

            # Stack into batch array and convert to numpy for embed_audio
            batch_tensor = torch.stack(padded_wavs).numpy()

            # Process entire batch at once
            batch_embeds = self.embed_audio(batch_tensor, self.audio_model)

            # Normalize each embedding in the batch
            norms = np.linalg.norm(batch_embeds, axis=-1, keepdims=True)
            normalized_embeds = batch_embeds / norms

            # Add each embedding from the batch
            for embed in normalized_embeds:
                all_embeddings.append(embed.reshape(1, -1))

        return np.vstack(all_embeddings)

    def get_text_embeddings(
        self,
        inputs: DataLoader[TextInput],
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = []
        for batch in tqdm(
            inputs, disable=not show_progress_bar, desc="Processing text batches"
        ):
            texts = batch["text"]
            features = self.clip_processor(
                text=texts, return_tensors="pt", padding=True, truncation=True
            )
            features = {k: v.to(self.device) for k, v in features.items()}

            with torch.no_grad():
                text_features = self.clip.get_text_features(**features)
                text_features = text_features / text_features.norm(dim=-1, keepdim=True)
            text_embeddings.append(text_features.cpu().detach().numpy())

        return np.vstack(text_embeddings)

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        **kwargs: Any,
    ) -> Array:
        text_embeddings = None
        audio_embeddings = None

        if "text" in inputs.dataset.features:
            text_embeddings = self.get_text_embeddings(inputs, **kwargs)
        if "audio" in inputs.dataset.features:
            audio_embeddings = self.get_audio_embeddings(inputs, **kwargs)

        if text_embeddings is not None and audio_embeddings is not None:
            if len(text_embeddings) != len(audio_embeddings):
                raise ValueError(
                    "The number of texts and images must have the same length"
                )
            fused_embeddings = text_embeddings + audio_embeddings
            return fused_embeddings
        elif text_embeddings is not None:
            return text_embeddings
        elif audio_embeddings is not None:
            return audio_embeddings
        raise ValueError


wav2clip_zero = ModelMeta(
    loader=Wav2ClipZeroShotWrapper,
    name="lyrebird/wav2clip",
    languages=["eng-Latn"],
    revision="no_revision",
    release_date="2022-03-15",
    modalities=["audio", "text"],
    n_parameters=163_000_000,  # wav2clip: 11.7M + CLIP: 151.3M ≈ 163M
    memory_usage_mb=622,  # wav2clip: 44.65MB + CLIP: 577.08MB ≈ 622MB
    max_tokens=None,
    embed_dim=512,
    license="mit",
    open_weights=True,
    framework=["PyTorch"],
    reference="https://github.com/descriptinc/lyrebird-wav2clip",
    similarity_fn_name="cosine",
    use_instructions=False,
    public_training_code="https://github.com/descriptinc/lyrebird-wav2clip",
    public_training_data="https://github.com/descriptinc/lyrebird-wav2clip#data",
    training_datasets=set(
        # "AudioSet": ["https://research.google.com/audioset/"],
        # "FreeSound": ["https://freesound.org/"],
        # "BBC Sound Effects": ["https://sound-effects.bbcrewind.co.uk/"],
    ),
    citation="""
@misc{wu2022wav2cliplearningrobustaudio,
  title={Wav2CLIP: Learning Robust Audio Representations From CLIP},
  author={Ho-Hsiang Wu and Prem Seetharaman and Kundan Kumar and Juan Pablo Bello},
  year={2022},
  eprint={2110.11499},
  archivePrefix={arXiv},
  primaryClass={cs.SD},
  url={https://arxiv.org/abs/2110.11499},
}""",
)



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, Any, Protocol, cast

from mteb._create_dataloaders import create_dataloader

from .evaluator import Evaluator

if TYPE_CHECKING:
    import numpy as np
    from datasets import Dataset
    from numpy.typing import NDArray
    from torch.utils.data import DataLoader
    from typing_extensions import Self

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.models import EncoderProtocol
    from mteb.types import Array, BatchedInput, EncodeKwargs

logger = logging.getLogger(__name__)


class SklearnModelProtocol(Protocol):
    def fit(
        self, X: Array, y: NDArray[np.integer | np.floating] | list[int | float]
    ) -> None: ...
    def predict(self, X: Array) -> NDArray[np.integer | np.floating]: ...
    def get_params(self) -> dict[str, Any]: ...
    def set_params(self, random_state: int, **kwargs: dict[str, Any]) -> Self: ...
    def score(
        self, X: Array, y: NDArray[np.integer | np.floating] | list[int | float]
    ) -> float: ...


class SklearnEvaluator(Evaluator):
    def __init__(
        self,
        train_dataset: Dataset,
        eval_dataset: Dataset,
        values_column_name: str,
        label_column_name: str,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        evaluator_model: SklearnModelProtocol,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.train_dataset = train_dataset
        self.eval_dataset = eval_dataset

        self.values_column_name = values_column_name
        self.label_column_name = label_column_name

        self.task_metadata = task_metadata
        self.hf_split = hf_split
        self.hf_subset = hf_subset
        self.evaluator_model = evaluator_model

    def create_dataloaders(
        self,
        encode_kwargs: EncodeKwargs,
        num_proc: int | None,
    ) -> tuple[DataLoader[BatchedInput], DataLoader[BatchedInput]]:
        dataloader_train = create_dataloader(
            self.train_dataset,
            self.task_metadata,
            input_column=self.values_column_name,
            num_proc=num_proc,
            **encode_kwargs,
        )
        dataloader_test = create_dataloader(
            self.eval_dataset,
            self.task_metadata,
            input_column=self.values_column_name,
            num_proc=num_proc,
            **encode_kwargs,
        )
        return dataloader_train, dataloader_test

    def __call__(  # type: ignore[override]
        self,
        model: EncoderProtocol,
        *,
        encode_kwargs: EncodeKwargs,
        test_cache: Array | None = None,
        train_cache: Array | None = None,
        num_proc: int | None = None,
    ) -> tuple[NDArray[np.integer | np.floating], Array]:
        """Classification evaluation by training a sklearn classifier on the embeddings of the training set and evaluating on the embeddings of the test set.

        Args:
            model: Encoder
            encode_kwargs: encode kwargs
            test_cache: embeddings of the test set, if already computed
            train_cache: embeddings of the train set, if already computed. Used for cross-validation.
            num_proc: number of processes to use

        Returns:
            Tuple of test predictions and embeddings

        """
        dataloader_train, dataloader_test = self.create_dataloaders(
            encode_kwargs=encode_kwargs,
            num_proc=num_proc,
        )

        logger.info("Running - Encoding samples...")
        if train_cache is None:
            train_cache = model.encode(
                dataloader_train,
                task_metadata=self.task_metadata,
                hf_split="train",
                hf_subset=self.hf_subset,
                **encode_kwargs,
            )
        if test_cache is None:
            test_cache = model.encode(
                dataloader_test,
                task_metadata=self.task_metadata,
                hf_split=self.hf_split,
                hf_subset=self.hf_subset,
                **encode_kwargs,
            )
            test_cache = cast("Array", test_cache)

        logger.info("Running - Fitting classifier...")
        y_train = self.train_dataset[self.label_column_name]
        self.evaluator_model.fit(train_cache, y_train)

        logger.info("Running - Evaluating classifier...")
        y_pred = self.evaluator_model.predict(test_cache)
        return y_pred, test_cache



from mteb.abstasks import AbsTaskClustering
from mteb.abstasks.task_metadata import TaskMetadata


class ESC50Clustering(AbsTaskClustering):
    label_column_name: str = "target"
    metadata = TaskMetadata(
        name="ESC50Clustering",
        description=(
            "The ESC-50 dataset contains 2,000 labeled environmental audio recordings "
            "evenly distributed across 50 classes (40 clips per class). "
            "These classes are organized into 5 broad categories: animal sounds, natural soundscapes & water sounds, "
            "human (non-speech) sounds, interior/domestic sounds, and exterior/urban noises. "
            "This task evaluates unsupervised clustering performance on environmental audio recordings."
        ),
        reference="https://huggingface.co/datasets/ashraq/esc50",
        dataset={
            "path": "mteb/esc50",
            "revision": "31ea534771aaf92c116698355e5cdc2ffda4bb6d",
        },
        type="AudioClustering",
        category="a2a",
        eval_splits=["train"],
        eval_langs=["zxx-Zxxx"],
        main_score="v_measure",
        date=("2023-01-07", "2023-01-07"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Environment Sound Clustering"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{piczak2015dataset,
  author = {Piczak, Karol J.},
  booktitle = {Proceedings of the 23rd {Annual ACM Conference} on {Multimedia}},
  date = {2015-10-13},
  doi = {10.1145/2733373.2806390},
  isbn = {978-1-4503-3459-4},
  location = {{Brisbane, Australia}},
  pages = {1015--1018},
  publisher = {{ACM Press}},
  title = {{ESC}: {Dataset} for {Environmental Sound Classification}},
  url = {http://dl.acm.org/citation.cfm?doid=2733373.2806390},
}
""",
    )
    input_column_name: str = "audio"
    max_fraction_of_documents_to_embed = None



from .cmteb_clustering import (
    CLSClusteringFastP2P,
    CLSClusteringFastS2S,
    CLSClusteringP2P,
    CLSClusteringS2S,
    ThuNewsClusteringFastP2P,
    ThuNewsClusteringFastS2S,
    ThuNewsClusteringP2P,
    ThuNewsClusteringS2S,
)

__all__ = [
    "CLSClusteringFastP2P",
    "CLSClusteringFastS2S",
    "CLSClusteringP2P",
    "CLSClusteringS2S",
    "ThuNewsClusteringFastP2P",
    "ThuNewsClusteringFastS2S",
    "ThuNewsClusteringP2P",
    "ThuNewsClusteringS2S",
]



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class SpeechCommandsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="SpeechCommands",
        description="A set of one-second .wav audio files, each containing a single spoken English word or background noise. To keep evaluation fast, we use a downsampled version of the original dataset by keeping ~50 samples per class for training.",
        reference="https://arxiv.org/abs/1804.03209",
        dataset={
            "path": "mteb/speech-commands-mini",
            "revision": "3ac713aa0829eeadda73182f38bbbd788d21254b",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2018-04-11", "2018-04-11"),  # v0.02 release date
        domains=["Speech"],
        task_subtypes=["Spoken Language Identification"],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@article{DBLP:journals/corr/abs-1804-03209,
  author = {Pete Warden},
  bibsource = {dblp computer science bibliography, https://dblp.org},
  biburl = {https://dblp.org/rec/journals/corr/abs-1804-03209.bib},
  eprint = {1804.03209},
  eprinttype = {arXiv},
  journal = {CoRR},
  timestamp = {Mon, 13 Aug 2018 16:48:32 +0200},
  title = {Speech Commands: {A} Dataset for Limited-Vocabulary Speech Recognition},
  url = {http://arxiv.org/abs/1804.03209},
  volume = {abs/1804.03209},
  year = {2018},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = False
